
(*  Title:      CorrectComponent.thy
    Author:     Ludovic Henrio 
                2008

    Note:           Lemmas and functions dealing with CorrectComponents and CorrectComponentWeak
                    definitions of a CorrectComponent are in GCM.thy
                        *)

header {* CorrectComponen*}

theory CorrectComponent imports BasicGCMLemmas Lemmas  
begin

lemma CorrectCompWeak[intro]: "CorrectComponent C \<Longrightarrow> CorrectComponentWeak C"
apply (unfold CorrectComponentWeak_def CorrectComponent_def);
apply auto;
done;

lemma corrCompStructListComp: "CorrectComponentStructure (Composite N  itfs subCp bindings s) \<Longrightarrow> CorrectComponentStructureList subCp";
apply auto;
done;


lemma corrCompStructListComp_rev[rule_format]: " CorrectComponentStructureList subCp \<longrightarrow> (\<forall>C\<in>set subCp. CorrectComponentStructure C)";
  
apply (induct_tac subCp);
apply auto;
done;
(* FORMATED lemma corrCompStructListComp_rev:

lemma corrCompStructListComp_rev:
        \<lbrakk>CorrectComponentStructureList ?subCp; ?x \<in> set ?subCp\<rbrakk>
        \<Longrightarrow> CorrectComponentStructure ?x*)

lemma CorrectComponentListComp[intro]: "CorrectComponentWeak (Composite N  itfs subCp bindings s) \<Longrightarrow> CorrectComponentWeakList subCp";
apply (unfold CorrectComponentWeakList_def CorrectComponentWeak_def);
apply (auto simp: corrCompStructListComp_rev);
done;

lemma CorrectComponentListComp_rev[rule_format]: " CorrectComponentWeakList subCp \<longrightarrow>(\<forall> C\<in> set subCp. CorrectComponentWeak C)"
apply (induct_tac subCp);
apply force;
apply (unfold CorrectComponentWeakList_def CorrectComponentWeak_def);
apply auto;
done;
(* FORMATTED lemma CorrectComponentListComp_rev:
        \<lbrakk>CorrectComponentWeakList ?subCp; ?x \<in> set ?subCp\<rbrakk> \<Longrightarrow> CorrectComponentWeak ?x *)

lemma correctCompWeak_distinct_name:" CorrectComponentWeak C \<Longrightarrow>  distinct (map getName (cpList C))"
apply (simp add:CorrectComponentWeak_def);
done;


(*FORMATED  correctCompWeak_distinct_name:
    CorrectComponentWeak ?C \<Longrightarrow> distinct (map getName (cpList ?C)) *)


lemma correctCompWeakList_distinct_name[rule_format]: " CorrectComponentWeakList CL\<Longrightarrow>distinct (map getName (cpListlist CL))"
  apply (simp add:CorrectComponentWeakList_def);
done;

lemma correctCompWeakList_distinct_localNames[rule_format]: " CorrectComponentWeakList CL\<Longrightarrow>distinct (map getName (CL))"
  apply (insert correctCompWeakList_distinct_name [of CL]);
  apply simp+;
sorry;

lemma correctCompWeak_UpdateCpSameName[rule_format]: 
  " CorrectComponentWeakList CL\<longrightarrow>N=getName C\<longrightarrow>CL^N\<noteq>None\<longrightarrow> {C'\<in>set (CL<-C).getName C'=N}={C}"
apply (induct_tac CL);
 apply simp;
apply clarify;
apply simp;
apply (frule correctCompWeakList_distinct_localNames);
apply (rule conjI);
 apply clarify;
 apply simp;
 apply (subgoal_tac "{C'. C' \<in> set list \<and> getName C' = getName C} = {}")
  apply force;
 apply simp+;
 apply (simp add: set_conv_nth);
 apply clarify;
 apply (rotate_tac -1)
 apply (drule sym);
 apply simp+;
 apply force;

apply (force simp: CorrectComponentWeakList_def)
done;
(* FORMATTED :   \<lbrakk>CorrectComponentWeakList ?CL; ?N = getName ?C; ?CL ^ ?N \<noteq> None\<rbrakk>
        \<Longrightarrow> {C' \<in> set (?CL <- ?C). getName C' = ?N} = {?C} *)

lemma CorrectComponentWeakList_simp[simp]: 
"CorrectComponentWeakList (C#CL) = (CorrectComponentWeak C \<and> CorrectComponentWeakList CL 
\<and> (set (RqIdList C) \<inter> (set (RqIdListList CL)) = {})
\<and> (getName ` (cpSet C) \<inter> getName ` (cpListset CL)  = {}))"
apply rule;
apply (simp add: CorrectComponentWeakList_def CorrectComponentWeak_def CorrectComponentStructure_def CorrectComponentStructureList_def cpSet_def cpListset_def)+;
done;


lemma CorrectCompWeakList_updatedCp[rule_format,intro]:
"CL^getName C' = Some C\<longrightarrow>CorrectComponentWeakList (CL<-C') \<longrightarrow>CorrectComponentWeak C'"
apply (induct_tac CL);
apply auto;
done;


lemma Behaviour_Pqueue[simp]:"Behaviour (s\<lparr>Pqueue:=Q\<rparr>) = Behaviour s"
apply (auto simp: Behaviour_def);
done;

lemma correctComponentWeak_Enqueue[simp]: "CorrectComponentWeak (C\<leftarrow>R) = (CorrectComponentWeak C \<and>id R \<notin> set (RqIdList C) \<and>(invokedItf R)\<in> dom (getItfs C) \<and> kind (the ((getItfs C) (invokedItf R))) = Server)"
apply (rule induct_Comp);
apply (unfold CorrectComponentWeak_def)
apply simp+;
apply rule;
apply simp+;
apply rule;
apply simp+;
done;

end;