(*  Title:      GCMSemanticsEagerHomeLemmas.thy
    Author:     Ludovic Henrio  & Khan Muhammad
                2009

    Note:     
*)
header {* GCM Semantics (EagerHome) proof of well formedness *}

theory EagerHomeFutureRegistration imports GCMSemanticsEagerHome  EagerHomeProperties
begin

lemma UpdFut_futdisappear: "f\<notin>set (snd v) \<Longrightarrow> f \<notin> set (snd (UpdateFuture f v v'))"
apply (unfold UpdateFuture_def);
apply auto;
done;
(*FORMATED UpdFut_futdisappear:
    ?f \<notin> set (snd ?v) \<Longrightarrow> ?f \<notin> set (snd (UpdateFuture ?f ?v ?v'))   *)

lemma UpdFut_futset: " set (snd (UpdateFuture f v v')) \<subseteq> set (snd v)\<union> set (snd v')"
apply (unfold UpdateFuture_def);
apply auto;
done;
(*FORMATED UpdFut_futdisappear:
    ?f \<notin> set (snd ?v) \<Longrightarrow> ?f \<notin> set (snd (UpdateFuture ?f ?v ?v'))   *)


theorem UpdFutRed_futdisappear: "S -[f, v, N]\<rightarrow>\<^sub>F S2  ,RL \<Longrightarrow> CorrectComponentWeak S \<longrightarrow>(S2^^N = Some C\<longrightarrow>f\<notin>set (snd v)\<longrightarrow>f\<notin> LocalReferencedRqs C)"
apply (erule Fut.induct);
(*2 compositeupdate*)
apply simp;
apply (rule); 
apply (rule impI)+;
apply (rotate_tac -2);
apply (drule sym);
apply simp+;
apply (rule);
apply (simp add: foldUn);
apply (erule disjE);
(*4 subgoals*)
apply (erule bexE);
apply  (frule_tac v'="(parameter b)" in UpdFut_futdisappear);
apply simp+;
apply (erule bexE);
apply  (frule_tac v'="(fValue b)" in UpdFut_futdisappear);
apply simp+;
(*2 subgoals*)
apply (rule impI)+;
apply (drule getRecSubCpList_conv_nth);
apply clarify;
apply simp+;
apply (drule_tac x=i in spec);
apply simp+;
apply (drule CorrectComponentListComp)
apply (drule_tac x="subCp!i" in  CorrectComponentListComp_rev)
apply simp+;

(*1 primitive update *)
apply (simp add: CorrectComponentWeak_def Behaviour_def);

apply (subgoal_tac "  Rep_Behaviours (behaviour s)\<in> Behaviours");
apply (simp add: Behaviours_def)
apply (erule conjE)+;apply (rotate_tac -2);
apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1,drule_tac x=s2 in spec);
 apply (drule_tac x="fst v" in spec);
 apply (drule_tac x="snd v" in spec);
 apply (drule_tac x="fst f" in spec);
 apply (drule_tac x="snd f" in spec);
apply clarify;
apply simp;
apply (erule conjE)+;
apply (simp only: foldUn);
apply (erule disjE);
(* 3 subgoals*)
apply (erule disjE);
apply (rotate_tac -4);
apply blast;
apply (erule bexE);
apply  (insert UpdFut_futdisappear);
apply (drule_tac x="(a,b)" in meta_spec)
apply (drule_tac x="(aa,ba)" in meta_spec)
apply (subgoal_tac "\<exists> x \<in> set (Pqueue s).  bb=x\<lparr>parameter := UpdateFuture (a, b) (aa, ba) (parameter x)\<rparr>");
(* 4 subgoals*)
apply (erule bexE);
apply simp;
apply (simp add: set_conv_nth);
apply (erule exE)+;
apply (rule_tac x=" (Pqueue s) ! i" in exI);
apply rule;
apply (rule_tac x="i" in exI);
apply blast;
apply (erule conjE);
apply (rotate_tac -2);
apply (simp);
apply (erule bexE);
(* 2 subgoals*)
apply (drule_tac x="(a,b)" in meta_spec)
apply (drule_tac x="(aa,ba)" in meta_spec)
apply (subgoal_tac "\<exists> x \<in> set (PcomputedResults s).  bb=x\<lparr>fValue := UpdateFuture (a, b) (aa, ba) (fValue x)\<rparr>");
apply (erule bexE);
apply simp;
apply (simp add: set_conv_nth);
apply (erule exE)+;
apply (rule_tac x=" (PcomputedResults s) ! i" in exI);
apply rule;
(* 3 subgoals*)
apply (rule_tac x="i" in exI);
apply blast;
apply (erule conjE);
apply (rotate_tac -2);
apply (simp);
apply (simp add: Rep_Behaviours);
done;
(*FORMATED UpdFutRed_futdisappear:
         ?S -[?f,?v,?N]\<rightarrow>\<^sub>F?S2.0,?RL  \<Longrightarrow>  CorrectComponentWeak ?S \<longrightarrow>
        ?S2.0 ^^ ?N = Some ?C \<longrightarrow> ?f \<notin> set (snd ?v) \<longrightarrow> ?f \<notin> LocalReferencedRqs ?C
  *)


constdefs RegisteredFuture:: "Fid \<Rightarrow> Name \<Rightarrow> Component \<Rightarrow> bool" 
  "RegisteredFuture f N S \<equiv>  \<exists>  C RL. (((S^^(snd f)) = Some C )\<and> (((getFutrMap C ) f)= Some RL) \<and> N \<in> set RL ) "

(*\<exists> Rl. (((getFutrMap ((cpList S)^(snd f))) f) = Some RL\<and>N\<in> set RL)"*)


constdefs LocalRegisteredFuturesComp:: "Component \<Rightarrow> Component \<Rightarrow> bool" 
(* returns true if all the futures of C are registered in the system S *)
"LocalRegisteredFuturesComp C S \<equiv>( \<forall>f\<in> LocalReferencedRqs C. RegisteredFuture f (getName C) S)"




lemma registeredFutures_O[rule_format]: "S \<turnstile> C1 -[i1,f,v]\<rightarrow>\<^sub>O C2 \<Longrightarrow>C1 \<in> cpSet S \<longrightarrow> (\<forall> C\<in>cpSet C1. LocalRegisteredFuturesComp C S)\<longrightarrow> 
  (\<forall> C'\<in>cpSet C2. \<forall> f'. ((f'\<in> LocalReferencedRqs C')\<longrightarrow> 
  (RegisteredFuture f' (getName C') S \<or>(f=f'\<and>C'=C2))))"

apply (erule comm.induct);
(* 2 Call *)
apply (subgoal_tac "LocalReferencedRqs (Primitive N itf (s\<lparr>PintState := s2\<rparr>)) \<subseteq> LocalReferencedRqs (Primitive N itf s)\<union>{f}");
apply (unfold LocalRegisteredFuturesComp_def  RegisteredFuture_def);
apply force;
apply (subgoal_tac "Rep_Behaviours (behaviour s)\<in> Behaviours");
apply (simp add: Behaviours_def);
apply (erule conjE)+;
apply (rotate_tac 4);
apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1, drule_tac x=s2 in spec);
 apply (simp add: Behaviour_def);
  apply (drule_tac x=i1 in spec);
   apply (drule_tac x="fst v" in spec);
   apply (drule_tac x="snd v" in spec);
   apply (drule_tac x="fst f" in spec);
   apply (drule_tac x="snd f" in spec);apply simp+;
apply rule;
apply (simp add: foldUn);
apply blast;
apply (simp add: Rep_Behaviours);
(*1 CompositeCall*)
apply (rule impI)+;apply rule;apply rule;apply rule;
apply simp;
apply (erule disjE)+;
apply simp+;
apply (erule disjE)
apply force;
apply (case_tac "f=f'")
apply (auto simp: foldUn);
done;

lemma future_in_V :"fa \<in> LocalReferencedRqs(C' \<leftarrow> \<lparr>Request.id = f, parameter = v, invokedItf = i2\<rparr>)\<Longrightarrow>fa \<in>LocalReferencedRqs C' \<or> fa\<in> set (snd v)"
apply (case_tac C');
apply auto;
apply (simp add:foldUn)+;
done;
(* FORMATED  future_inRef_value:
        ?fa \<in> LocalReferencedRqs (?C' \<leftarrow> \<lparr>Request.id = ?f, parameter = ?v, invokedItf = ?i2.0\<rparr>) \<Longrightarrow> ?fa \<in> LocalReferencedRqs ?C' \<or> ?fa \<in> set (snd ?v)*)

lemma future_in_parameter[rule_format]: "S \<turnstile> C1 -[i1,f,v]\<rightarrow>\<^sub>O C2 \<Longrightarrow> f'\<in>set (snd v) \<longrightarrow> f'\<in>LocalReferencedRqs C1"
apply (erule comm.induct);
apply rule;
apply (subgoal_tac "  Rep_Behaviours (behaviour s)\<in> Behaviours");
(*3*)
 apply (simp add: Behaviours_def);
 apply clarify;apply (rotate_tac 5);
 apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1, drule_tac x=s2 in spec);
 apply (rotate_tac -1, drule_tac x=i1 in spec);
 apply (rotate_tac -1, drule_tac x=a in spec);
 apply (rotate_tac -1, drule_tac x=b in spec);
 apply (rotate_tac -1, drule_tac x=aa in spec);
 apply (rotate_tac -1, drule_tac x=ba in spec);
 apply (simp add: Behaviour_def);
 apply clarify;
 apply (rotate_tac -3);apply (simp add:foldUn)+;
 apply blast;
apply (simp add: Rep_Behaviours)
apply (simp add:foldUn)+;
done

declare cpSetcomposite[simp del]
declare cpSetcomposite_set[simp]

lemma registeredFutures_R[rule_format]: "S\<turnstile> C1 \<rightarrow>\<^sub>R C2 ,RL \<Longrightarrow>C1 \<in> set(cpList S) \<longrightarrow> (\<forall> C\<in>cpSet C1. LocalRegisteredFuturesComp C S)\<longrightarrow> 
  (\<forall> C'\<in>cpSet C2 . (\<forall> f\<in> LocalReferencedRqs C'.
  (RegisteredFuture f (getName C') S \<or>(f,(getName C')) \<in>(set RL))))"
apply (erule red.induct);
(* 7 TAU *)
apply (subgoal_tac "LocalReferencedRqs (Primitive N itf (s\<lparr>PintState := s2\<rparr>)) \<subseteq> LocalReferencedRqs (Primitive N itf s)")
apply (unfold LocalRegisteredFuturesComp_def  RegisteredFuture_def);
apply force;
apply (subgoal_tac "  Rep_Behaviours (behaviour s)\<in> Behaviours");
apply (simp add: Behaviours_def);
 apply clarify;apply (rotate_tac 1);
 apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1, drule_tac x=s2 in spec);
apply (simp add: Behaviour_def);
apply clarify;
apply (simp add: foldUn);
apply (rotate_tac -4);
apply blast;
apply (simp add: Rep_Behaviours)

(* 6 ENDSERVICE *)
apply (subgoal_tac "LocalReferencedRqs (Primitive N itf
              (s\<lparr>PintState := s2,
                   PcomputedResults := PcomputedResults s @ [\<lparr>fid = fa, fValue = v\<rparr>]\<rparr>))
                     \<subseteq> LocalReferencedRqs (Primitive N itf s)")
apply force;
apply (subgoal_tac "  Rep_Behaviours (behaviour s)\<in> Behaviours");
apply (simp add: Behaviours_def);
 apply clarify;apply (rotate_tac 6);
 apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1, drule_tac x=s2 in spec);
apply (simp add: Behaviour_def);
 apply (drule_tac x=aa in spec);
 apply (drule_tac x=ba in spec);
 apply (drule_tac x=a in spec);
 apply (drule_tac x=b in spec);
apply clarify;
apply rule;
apply clarify;
apply (simp add: foldUn);
apply (rotate_tac -5);
apply blast;
apply clarify;
apply (simp add: foldUn);
apply (rotate_tac -4);
apply blast;
apply (simp add: Rep_Behaviours);

(* 5 NewService *)
apply (subgoal_tac "LocalReferencedRqs  (Primitive N itf
                 (s\<lparr>PintState := s2, Pqueue := queue\<rparr>))
                     \<subseteq> LocalReferencedRqs (Primitive N itf s)");
apply force;
apply (subgoal_tac "  Rep_Behaviours (behaviour s)\<in> Behaviours");
 apply (simp add: Behaviours_def);
 apply clarify;
 apply (rotate_tac 4);
 apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1, drule_tac x=s2 in spec);
 apply (simp add: Behaviour_def);
  apply (drule_tac x=i in spec);
  apply (drule_tac x="fst (parameter R)" in spec);
  apply (drule_tac x="snd (parameter R)" in spec);
  apply (drule_tac x="fst (Request.id R)" in spec);
  apply (drule_tac x="snd (Request.id R)" in spec);
apply (simp add: foldUn);
apply (rotate_tac -5);
apply blast;
apply (simp add: Rep_Behaviours);

(* 4 Hierarchy *)
apply rule;
apply rule;apply rule;
apply simp+;
apply (erule disjE);
apply (subgoal_tac "LocalReferencedRqs  (Composite N0 itf (subCp <- c2) bindings s)
                     = LocalReferencedRqs (Composite N0 itf subCp bindings s)");
apply (simp add: cpListset_Un)+;
apply rule;
apply (erule bexE);
apply (subgoal_tac "C'\<in> (cpListset (subCp <- c2))");
apply (drule upd_setCp);
apply (erule disjE);
(*6 in subCp *)
apply (erule conjE)+;
apply (drule_tac x=C' in spec);
apply (erule impE);
apply (fold cpSet_def);
apply (rule_tac C'="Composite N0 itf subCp bindings s" in cpSetcpSet);
apply simp+;
apply (drule getCp_inlist);
apply (rule disjI2);
apply force;
apply simp; 
apply (force)
(*5 in c2*)
apply (erule impE);
apply (drule getCp_inlist);
apply (rule_tac C'="Composite N0 itf subCp bindings s" in cpSetcpSet);
apply (force );
apply (force simp: cpSetFirst);
apply force;
apply force;

(*3 CommBrothers*)
apply simp;
apply rule;
apply rule;
apply rule;
apply rule;
apply (erule conjE, erule bexE);
apply rule;

(*4 introduce Big subgoal *)
apply (subgoal_tac "\<forall>C\<in>(cpListset (subCp <- c2)).
           \<forall>fz\<in>LocalReferencedRqs C. (\<exists>Ca. S ^^ snd fz = Some Ca \<and> (\<exists>RL. getFutrMap Ca fz = Some RL \<and> getName C \<in> set RL))\<or>  (fz= f \<and> getName C = N)")
apply (drule upd_setCpSimple);
apply (erule disjE)

(*5*)
apply (rotate_tac -2);
apply (drule_tac x=C'a in bspec);
apply force;
(*5*)
apply (rotate_tac -1);
apply (drule_tac x=fa in bspec);
apply blast;
apply force;

(*4 in the receiver *)
apply simp+;
apply (case_tac C');
(*5 primitive *)
apply (simp)+;
apply (rotate_tac -3,drule_tac x=C' in bspec);
apply (drule getCp_inlist)+;
apply (rule CL_to_list);
apply force;
apply simp+;
apply (case_tac "fa\<in> set (snd v)")
apply (rule disjI2);
apply (rule disjI2);
apply (drule getCp_inlist)+;
apply force;
apply (rotate_tac -2, drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply force;

(* 4 composite starts almost like prim *)
apply simp+;
apply (erule disjE);
apply (rotate_tac -3,drule_tac x=C' in bspec);
apply (drule getCp_inlist)+;
apply (rule CL_to_list);
apply force;
apply simp+;
apply (case_tac "fa\<in> set (snd v)")
apply (rule disjI2);
apply (rule disjI2);
apply (drule getCp_inlist)+;
apply force;
apply (rotate_tac -2, drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply force;

(*4 inside composite *)
apply (erule bexE);
apply (rotate_tac -3,drule_tac x=C'a in bspec);
apply (drule getCp_inlist)+;
apply force;
apply force;
(* 3 prove Big subgoal *)
apply rule;apply rule;
apply (drule upd_setCp);
apply (erule disjE);
(* in subCp *)

apply (drule_tac x=Ca in spec);
apply (force);

(* 3  in the sender *)
apply (frule_tac f'=fz  in registeredFutures_O);
apply (drule getCp_inlist)+;
apply (subgoal_tac "C\<in>cpSet (Composite N0 itf subCp bindings s)");
apply (rule_tac C'="(Composite N0 itf subCp bindings s)" in cpSetcpSet);
apply simp+;
apply (force);
apply (unfold RegisteredFuture_def LocalRegisteredFuturesComp_def);
apply simp+;
apply rule;
apply (drule_tac x=Cb in spec);
apply (drule getCp_inlist)+;
apply force;
apply simp+;
apply (drule comm_maintains_name);
apply (force simp: getCp_name);

(*2 CommParent *)
apply rule;
apply rule;
apply rule;
apply rule;
apply (simp);
apply (erule disjE);
apply simp+;
apply (erule conjE);
apply (case_tac "fa\<in> set (snd v)");

(*4*)
apply (rule disjI2);
apply (rule disjI2);
apply (drule getCp_inlist)+;
apply force;
apply (drule_tac x=fa in bspec);
apply (simp add: foldUn);
apply simp;

(*2 in subcomponents*)
apply (erule bexE)
apply (drule upd_setCpSimple);
apply (erule disjE);
apply force;
apply (frule registeredFutures_O);
apply (rule_tac C'="Composite N0 itf subCp bindings s" in  cpSetcpSet);
apply (simp add: getCp_inlist)+;
apply (drule getCp_inlist)+; 
apply (force);
apply simp;
apply (unfold LocalRegisteredFuturesComp_def RegisteredFuture_def);
apply rule;
apply simp+;
apply (erule conjE);
apply (drule_tac x=Ca in spec);
apply (drule getCp_inlist)+;
(*5*) 
apply force;
apply force;
apply force;
apply (erule conjE)+;
apply (drule comm_maintains_name);
apply (force simp: getCp_name);

(*1 CommChild*)
apply rule;
apply rule;
apply rule;
apply rule;
apply (simp);
apply (erule disjE);
apply simp+;
apply (erule conjE);
apply (case_tac "fa=f");
apply (simp add: foldUn)+;

(*1*)
apply (erule bexE);
apply (drule upd_setCpSimple);
apply (erule disjE);
apply force;
(*1*)
apply (case_tac C');
apply (simp add: cpSet_def);
apply (subgoal_tac "fa\<in> LocalReferencedRqs (C' \<leftarrow> \<lparr>Request.id = i, parameter = parameter R, invokedItf = i2\<rparr>)")
apply (drule future_in_V);
apply simp+;
apply (erule disjE);
apply (drule getCp_inlist)+;
apply force;
(*3*)
apply (drule  getCp_name);
apply force;
apply simp+;

(*1*)
apply (erule disjE);
apply (subgoal_tac "fa\<in> LocalReferencedRqs (C' \<leftarrow> \<lparr>Request.id = i, parameter = parameter R, invokedItf = i2\<rparr>)")
apply (drule future_in_V);
apply simp+;
apply (erule disjE);
apply (drule getCp_inlist)+;
apply force;
(*3*)
apply (drule  getCp_name);
apply force;
apply simp+;
apply (drule getCp_inlist)+;
apply force;
done;

declare cpSetcomposite_set[simp del]
declare cpSetcomposite[simp]

constdefs GlobalRegisteredFuturesComp:: "Component \<Rightarrow> bool"
"GlobalRegisteredFuturesComp  S \<equiv>( \<forall> C \<in> (cpSet S).(LocalRegisteredFuturesComp C S))"


lemma locateList_locate:
"C \<in> set (locateComponentandRegisterFutureList CL N Fid N') = (\<exists> C'\<in> set CL. C=(locateComponentandRegisterFuture C' N Fid N'))"
apply (induct_tac CL);
apply auto;
done;



lemma locate_component_diff_name_prim[rule_format]:
" Primitive N0 itfs s \<in> cpSet (locateComponentandRegisterFuture S N Fid N')\<longrightarrow>N0\<noteq>N\<longrightarrow>Primitive N0 itfs s \<in> cpSet S"
apply (rule induct_Comp);
apply (auto simp: locateList_locate cpListset_Un);
apply (force);
done;

lemma locate_component_eq_name_prim[rule_format]:
" CorrectComponentWeak S \<longrightarrow> 
Primitive N itfs s \<in> cpSet (locateComponentandRegisterFuture S N Fid N')\<longrightarrow>
(\<exists> s' oldFM. (Primitive N itfs s'\<in> cpSet S)\<and>s'=s\<lparr>PfutrMap:=oldFM \<rparr> 
\<and> (\<forall> f. if f=Fid then
N'#optionlist2list (oldFM f)=optionlist2list((PfutrMap s) f)
else (oldFM f)=((PfutrMap s) f)))"
apply (rule induct_Comp);
apply (auto);
apply (rule_tac x="(PfutrMap s)" in exI);
apply auto;
apply (subgoal_tac "CorrectComponentWeak(Composite N itfsa cplist bindings
             (sa\<lparr>CfutrMap := CfutrMap sa(Fid \<mapsto>
                   N' # optionlist2list (CfutrMap sa Fid))\<rparr>))" );
apply (rotate_tac -1);
apply (drule correctCompWeak_distinct_name);
apply auto;
(*3*)
apply (subgoal_tac "(Primitive  N itfs s \<in> (cpListset cplist))")
apply (unfold image_def);
apply simp;
apply (rotate_tac 2)
apply (drule_tac x= "Primitive N itfs s" in bspec)
apply (fold cpListset_def)
apply (auto simp: cpListset_Un);
defer;
(*2 correctcomponent later*)
apply (simp add: locateList_locate)
apply clarify;
apply (drule_tac x=C'a in bspec);
apply simp+;
apply (erule impE);
apply (rule_tac C="(Composite Na itfsa cplist bindings sa)" in SubComponent_CorrectComponentWeak);
apply (rule cpSetcomposite_rev);
apply auto;
apply (force simp: cpSet_def);

 (*1  correctcomponnt*)
apply (unfold CorrectComponentWeak_def)
apply auto;
done;


lemma locate_component_diff_name_comp_rec[rule_format]:
"((\<forall>C \<in> (cpSet C0). (getName C \<noteq> N) )
\<longrightarrow> (C0 = (locateComponentandRegisterFuture C0 N Fid N'))) \<and>
((\<forall>C \<in> (cpListset CL). (getName C \<noteq> N) )
\<longrightarrow> (CL = (locateComponentandRegisterFutureList CL N Fid N')))
"
apply (induct_tac C0 and CL);
apply auto;
done;


lemma locate_component_diff_name_comp:
"\<forall>C \<in> (cpSet C0). (getName C \<noteq> N) 
\<Longrightarrow> C0 = (locateComponentandRegisterFuture C0 N Fid N')"
by (auto simp: locate_component_diff_name_comp_rec)

lemma locateComponent_getNameEquals[simp]:
"getName (locateComponentandRegisterFuture C N f N') = getName C"
apply (rule induct_Comp);
apply auto;
done;

lemma locateComponentandRegisterFutureList_length[rule_format,simp]:
"length (locateComponentandRegisterFutureList CL N f N') = length CL"
apply (induct_tac CL);
apply auto;
done;

lemma locateComponentandRegisterFutureList_nth[rule_format,simp]:
" \<forall> i<length CL. (locateComponentandRegisterFutureList CL N f N')!i = (locateComponentandRegisterFuture (CL!i) N f N')"
apply (induct_tac CL);
apply auto;
apply (case_tac i)
apply auto;
done;
(*  ?i < length ?CL \<Longrightarrow>
        locateComponentandRegisterFutureList ?CL ?N ?f ?N' ! ?i =
        locateComponentandRegisterFuture (?CL ! ?i) ?N ?f ?N' *)

lemma locate_component_eq_name_set[rule_format]:
" CorrectComponentWeak S \<longrightarrow> 
Composite N itfs sub bind s \<in> cpSet (locateComponentandRegisterFuture S N Fid N')\<longrightarrow>
(\<exists> s' oldFM. (Composite N itfs sub bind s'\<in> cpSet S)\<and>s'=s\<lparr>CfutrMap:=oldFM \<rparr> 
\<and> (\<forall> f. if f=Fid then
N'#optionlist2list (oldFM f)=optionlist2list((CfutrMap s) f)
else (oldFM f)=((CfutrMap s) f)))"
apply (rule induct_Comp);
apply simp+;
apply (rule);
apply clarify
apply rule;
apply clarify;
(*3*)
apply (rule_tac x=s in exI);
apply simp+;
apply (rule_tac x="(CfutrMap s)" in exI);
apply simp+;
apply clarify;
(*2*)
apply (frule_tac i'=itfs in CorrectComponentWeak_same_name_eq)
apply force;
apply (insert cp_nonRec[of N itfs sub bind s])
apply (force);
(*1*)
apply (simp add: set_conv_nth cpListset_Un);
apply clarify
apply (frule_tac N=N and f=Fid and N'=N' in locateComponentandRegisterFutureList_nth);
apply simp+;
apply (drule_tac x="cplist!i" in spec);
apply (erule impE);
apply (rule_tac x=i in exI);
apply simp+;
(*1*)
apply (erule impE);
apply (rule  CorrectComponentListComp_rev); 
apply auto;
apply force;
done;

lemma locate_set: 
" {C \<in> set (locateComponentandRegisterFutureList list N f N'). getName C = Name1} = {locateComponentandRegisterFuture C N f N'|C . C \<in> set list\<and> getName C = Name1}"
apply (induct_tac list);
apply (auto simp: locateComponent_getNameEquals);
done;
 
lemma locate_RqIdList_induct: 
"(RqIdList (locateComponentandRegisterFuture C N f N') = RqIdList C )\<and>
(RqIdListList (locateComponentandRegisterFutureList CL N f N') = RqIdListList CL)"
apply (induct_tac C and CL);
apply auto;
done;

lemma locate_RqIdList_Cp: 
"RqIdList (locateComponentandRegisterFuture C N f N') = RqIdList C "
by (insert locate_RqIdList_induct, auto);

lemma locate_RqIdList_CL: 
"RqIdListList (locateComponentandRegisterFutureList CL N f N') = RqIdListList CL"
by (insert locate_RqIdList_induct, auto);


lemma locate_getNameIm_induct: 
"(getName ` cpSet (locateComponentandRegisterFuture C N f N') = getName ` (cpSet C) )\<and>
(getName ` cpListset (locateComponentandRegisterFutureList CL N f N') = getName ` (cpListset CL))"
apply (induct_tac C and CL);
apply (auto);
done;


lemma locate_getNameIm_Cp: 
"getName ` cpSet (locateComponentandRegisterFuture C N f N') = getName ` (cpSet C) "
by (insert locate_getNameIm_induct, auto);

lemma locate_getNameIm_CL: 
"getName ` cpListset (locateComponentandRegisterFutureList CL N f N') = getName ` (cpListset CL)"
by (insert locate_getNameIm_induct, auto);

lemma locate_getCp[intro,rule_format]:
"CL^Na=Some C \<longrightarrow>(locateComponentandRegisterFutureList CL N f N')^Na=Some (locateComponentandRegisterFuture C N f N')"
apply (induct_tac CL);
apply auto;
done;


lemma locate_CorrectComponentWeak_induct[rule_format]:
"( CorrectComponentWeak S \<longrightarrow> CorrectComponentWeak (locateComponentandRegisterFuture S N f N')) \<and>
( CorrectComponentWeakList CL \<longrightarrow> CorrectComponentWeakList (locateComponentandRegisterFutureList CL N f N'))"
apply (induct_tac S and CL);
apply (simp+);
apply (simp add: CorrectComponentWeak_def PintState_update_def PintState_upd_def Behaviour_def);
apply clarify;
apply (simp(no_asm) add: CorrectComponentWeak_Composite);
apply (rule conjI);
apply clarify;
apply rule;
apply force;
apply simp;
(*4*)
apply (simp add: cpListset_def CorrectComponentWeak_def);
apply rule;
apply clarify;
apply blast;
apply blast;

(*3*)
apply clarify;
apply rule;
apply (drule CorrectComponentListComp);
apply simp+;
apply rule;
apply (simp add: locate_getNameIm_CL)
apply (simp add: CorrectComponentWeak_def  cpListset_def);
(*3*)
apply rule;
apply (simp add: CorrectComponentWeak_def);
apply clarify;
apply (drule_tac x=b in bspec);
apply force;
apply rule;
apply (case_tac "src b");
apply simp+;
apply (case_tac "(list  ^ Name1)")
apply simp+;
apply clarify;
apply (drule_tac N=N and f=f and N'=N' in locate_getCp);
apply (case_tac a);
apply simp+;
(*4*)
apply (case_tac "dest b");
apply simp+;
apply (case_tac "(list  ^ Name1)")
apply simp+;
apply clarify;
apply (drule_tac N=N and f=f and N'=N' in locate_getCp);
apply (case_tac a);
apply simp+;

(*3*)
apply (simp add: CorrectComponentWeak_def);
apply clarify;
apply rule;
apply blast;
apply (simp add: locate_RqIdList_CL)
apply blast;
apply (simp add: CorrectComponentWeak_def  locate_RqIdList_Cp locate_RqIdList_CL locate_getNameIm_CL locate_getNameIm_Cp)+;
done;


lemma locate_CorrectComponentWeak:
"CorrectComponentWeak S \<Longrightarrow> CorrectComponentWeak (locateComponentandRegisterFuture S N f N')"
apply(insert locate_CorrectComponentWeak_induct[of S]);
apply auto;
done;
lemma locate_CorrectComponentWeakList:
"CorrectComponentWeakList CL \<Longrightarrow> CorrectComponentWeakList (locateComponentandRegisterFutureList CL N f N')"
apply(insert locate_CorrectComponentWeak_induct);
apply auto;
done;

lemma locate_registeredFuture[rule_format]: "CorrectComponentWeak S \<longrightarrow>snd f\<in> getName ` cpSet S \<longrightarrow>RegisteredFuture f N (locateComponentandRegisterFuture S (snd f) f N)"
apply (rule_tac Comp=S in induct_Comp);
apply (auto simp: RegisteredFuture_def cpListset_Un);
apply (drule_tac x=C' in bspec);apply simp+;
apply (erule impE);
apply (rule CorrectComponentListComp_rev);
apply auto;
apply (rule_tac x=C in exI);apply simp;
apply (drule CorrectComponentListComp);
apply (frule CorrectComponentListComp_rev);
apply simp+;
(*1*)
apply (drule_tac N="getName x" and f=f and N'=N in locate_CorrectComponentWeak);
apply (drule_tac N="getName x" and f=f and N'=N in locate_CorrectComponentWeakList);
apply (drule_tac N="getName x" and C'=C in getRecSubCp_Some)
apply (drule_tac N="getName x" and C'=C in getRecSubCpList_Some)
apply simp+;
apply (rule  setcpSet_cpListset);
defer;apply force;
apply (auto simp: set_conv_nth locateComponentandRegisterFutureList_nth);
apply (rule_tac x=i in exI);
apply auto;
apply (rule_tac x=i in exI);
apply auto;
done;

lemma locate_referencedRqs_induct: "ReferencedRqs C = ReferencedRqs (locateComponentandRegisterFuture C N f N') \<and>
ReferencedRqsList CL = ReferencedRqsList (locateComponentandRegisterFutureList CL N f N')"
apply (induct_tac C and CL);
apply simp+;
done;

lemma locate_referencedRqs_Cp[rule_format,simp]: "ReferencedRqs (locateComponentandRegisterFuture C N f N') = ReferencedRqs C"
by (insert locate_referencedRqs_induct, auto);


lemma localRef_incGlobalRefs_induct: 
"(\<forall> C.  (C\<in> cpSet S \<longrightarrow> LocalReferencedRqs C \<subseteq> ReferencedRqs S )) \<and>
(\<forall> C.  (C\<in> cpListset CL \<longrightarrow> LocalReferencedRqs C \<subseteq> ReferencedRqsList CL ))"
apply (induct_tac S and CL);
apply simp+;
apply (auto simp: foldUn);
done;

lemma localRef_incGlobalRefs_Cp: 
"C\<in> cpSet S \<Longrightarrow> LocalReferencedRqs C \<subseteq> ReferencedRqs S"
by (insert localRef_incGlobalRefs_induct, auto)


(* should we define a consts for correct cp weak+ snd of referenced futures exists ???????????*)

lemma locate_cpSet_induct: 
"(CorrectComponentWeak S \<longrightarrow>C \<in> cpSet (locateComponentandRegisterFuture S N f N')\<longrightarrow> (\<exists> C'. (S^^(getName C))=Some C' \<and> C= (locateComponentandRegisterFuture C' N f N'))) \<and>
(CorrectComponentWeakList CL \<longrightarrow>C \<in> cpListset (locateComponentandRegisterFutureList CL N f N')\<longrightarrow> (\<exists> C'. (getRecSubCpList CL (getName C))=Some C' \<and> C= (locateComponentandRegisterFuture C' N f N')))
"
apply (induct_tac S and CL);
(*4 primitive*)
apply simp;
(*3 composite *)
apply clarify;
apply (subgoal_tac "CorrectComponentWeak C");
apply simp;
apply (subgoal_tac  "C \<in> cpSet (Composite Name fun1
  (if Name = N then list else (locateComponentandRegisterFutureList list N f N'))
  fun2
  (if Name = N then (CompState_ext_type\<lparr>CfutrMap := CfutrMap CompState_ext_type(f \<mapsto> N' # optionlist2list (CfutrMap CompState_ext_type f))\<rparr>) else CompState_ext_type))");
apply simp+;
(*5 in a subcp *)
apply (erule disjE);
apply simp+;
apply (split split_if_asm) 
apply simp+;
apply clarify;
apply simp+;
(*6*)

apply (subgoal_tac " \<forall> C'\<in> cpSet C. getName C'\<noteq> N");
apply (subgoal_tac " getName C\<noteq> N");
apply simp+;
apply (drule_tac N'=N' and Fid=f in locate_component_diff_name_comp);
apply (rule_tac x=C in exI);
apply simp+;
apply (rule getRecSubCpList_getName);
apply force;
apply force;
apply (force)
apply (drule correctCompWeak_distinct_name)+;
apply simp+;
apply clarify;
apply (subgoal_tac "getName C' \<in> getName ` set (cpListlist list) ");
apply force;
apply (subgoal_tac "C' \<in> (cpListset list)");
apply (force simp: cpListset_def);
apply (drule cpListset_cpSet)
apply clarify
apply (rule setcpSet_cpListset)
apply simp;

(*6*)
apply (erule_tac C'=C in cpSetcpSet);
apply force;
apply (simp )+;
apply (erule impE);
apply auto;

(*4*)
apply (rotate_tac 1,frule correctCompWeak_distinct_name);
apply simp+;
apply clarify;
apply (subgoal_tac "getName C' \<in> getName ` set (cpListlist list)")
apply force;
apply (subgoal_tac "C' \<in> (cpListset list)");
apply (simp add: cpListset_def);
apply (drule CorrectComponentListComp)
apply (drule_tac C'=C' in getRecSubCpList_Some)
apply force;

(*3 prove correctcomponentweak*)
apply (split split_if_asm)
apply simp+;
apply (erule disjE);
apply (drule_tac N=N in locate_CorrectComponentWeak);
apply force;
apply clarify;
apply (rule SubComponent_CorrectComponentWeak)
apply auto;

(*3 correctcomponentweak again*)
apply (drule_tac N=N in locate_CorrectComponentWeak);
apply simp+;

(*2*)
apply (rule locate_CorrectComponentWeak);
apply (drule CorrectComponentListComp)
apply (frule_tac CL=list and C'=C' and N = "getName C'" in getRecSubCpList_Some)
apply clarify;
apply (drule cpListset_cpSet)
apply clarify;
apply (rule SubComponent_CorrectComponentWeak)
apply simp+;
apply (rule CorrectComponentListComp_rev)
apply simp+;
(*1*)
apply (subgoal_tac "(Component ^^ getName C') = None");
apply simp+;
apply (erule contrapos_pp);
apply clarify;
apply (drule_tac C=Component and C'=y and N = "getName C'" in getRecSubCp_Some)
apply (frule_tac CL=list and C'=C' and N = "getName C'" in getRecSubCpList_Some)
apply auto;
done;

lemma locate_cpSet: 
"\<lbrakk>CorrectComponentWeak S ; C \<in> cpSet (locateComponentandRegisterFuture S N f N')\<rbrakk>
\<Longrightarrow> \<exists> C'. (S^^(getName C))=Some C' \<and> C= (locateComponentandRegisterFuture C' N f N')"
by (insert locate_cpSet_induct, auto);

lemma locate_localrefs[simp]: 
"LocalReferencedRqs (locateComponentandRegisterFuture C N f N') = LocalReferencedRqs C"
apply (case_tac C)
apply (auto);
done;

lemma locate_registeredFuture_extends_induct:
"(CorrectComponentWeak C \<longrightarrow>  RegisteredFuture f N C\<longrightarrow> (RegisteredFuture f N (locateComponentandRegisterFuture C N1 f' N2))) \<and>
(CorrectComponentWeakList CL \<longrightarrow> (\<forall> C \<in> set CL. (RegisteredFuture f N C \<longrightarrow> ( \<exists> C' \<in> set (locateComponentandRegisterFutureList CL N1 f' N2). RegisteredFuture f N C')))) "
apply (induct_tac C and CL);
apply (force simp: RegisteredFuture_def);
apply clarify;
apply simp+;
apply rule;
apply (force simp: RegisteredFuture_def);
apply clarify;
apply (drule CorrectComponentListComp);
apply simp+;
(*3*)
apply (simp add: RegisteredFuture_def);
apply rule;
apply (force);
apply clarify;
apply (split split_if_asm);
apply force;
apply (erule impE);
apply (drule getRecSubCpList_conv_nth);
apply force;
apply auto;
(*1*)
apply (rule_tac x=Ca in exI);
apply simp+;
apply (drule_tac N=N1 and f=f' and N'=N2 in locate_CorrectComponentWeakList);
apply (frule_tac C'=Ca and N="snd f" in getRecSubCpList_Some);
apply simp+;
apply rule;
apply (rule setcpSet_cpListset);
apply simp+;
apply (rule getRecSubCp_Some_pre1_Cp);
apply simp+;
apply (drule getRecSubCp_Some_pre2_Cp);
apply simp+;
done;

lemma locate_registeredFuture_extends:
"\<lbrakk>CorrectComponentWeak C; RegisteredFuture f N C\<rbrakk>\<Longrightarrow> RegisteredFuture f N (locateComponentandRegisterFuture C N1 f' N2)"
by (insert locate_registeredFuture_extends_induct, auto);

lemma futures_are_registered_in_S2[rule_format] : " \<forall> S2. (CorrectComponentWeak S2\<and>(\<forall> f. f\<in> ReferencedRqs S2\<longrightarrow> (snd f \<in> getName ` (cpSet S2)) ) )
\<longrightarrow> (\<forall> C'\<in> cpSet S2 . (\<forall> f\<in> LocalReferencedRqs C'.  (RegisteredFuture f (getName C') S2 \<or>(f,(getName C')) \<in>(set RL)))) \<longrightarrow> GlobalRegisteredFuturesComp (RegisterListOfFutures S2 RL)"
apply (induct_tac RL);
apply clarify;
apply (simp add:  RegisterListOfFutures_def GlobalRegisteredFuturesComp_def  LocalRegisteredFuturesComp_def);
apply simp+;
apply (unfold GlobalRegisteredFuturesComp_def  LocalRegisteredFuturesComp_def );
apply (rule allI);
apply clarify;
apply (drule_tac x="(locateComponentandRegisterFuture S2 (snd (fst ((a, b), ba))) (fst ((a, b), ba))
                      (snd ((a, b), ba)))" in spec);
apply simp+;
apply (rotate_tac 3);
apply (erule impE);
apply rule;
apply (rule locate_CorrectComponentWeak);
apply (simp add: locate_getNameIm_Cp)+;
apply (erule impE);
apply clarify;
apply auto;
(*1*)
apply (case_tac "(ab,bc) = (a,b)");
apply simp+;
apply (case_tac "ba = getName C'");
apply simp+;
apply (drule_tac N="getName C'" and f="(a,b)" in locate_registeredFuture);
apply (drule_tac x=a in spec);
apply (drule_tac x=b in spec);
apply (simp add: locate_getNameIm_Cp)+;
apply (erule impE);
apply (rotate_tac 3,drule localRef_incGlobalRefs_Cp)
apply simp+;
apply force;
apply simp+;
apply (frule_tac locate_cpSet);
apply simp+;
apply clarify;
apply simp;
(*2*)
apply (frule_tac N="getName C'a" and C'=C'a in CpList.getRecSubCp_Some);
apply (drule_tac x=C'a in bspec);
apply simp+;
apply (drule_tac x="(a,b)" in bspec);
apply simp+;
apply (rule locate_registeredFuture_extends)
apply simp+;
apply (frule_tac locate_cpSet);
apply simp+;
(*1*)
apply clarify;
apply simp;
apply (frule_tac N="getName C'a" and C'=C'a in CpList.getRecSubCp_Some);
apply (drule_tac x=C'a in bspec);
apply simp+;
apply (drule_tac x="(ab,bc)" in bspec);
apply simp+;
apply (rule locate_registeredFuture_extends)
apply simp;
apply force;
done;

(*thm_deps futures_are_registered_in_S2 locate_registeredFuture_extends locate_cpSet locate_component_eq_name_set registeredFutures_R UpdFutRed_futdisappear; *)
lemma  registeredFutures_F_general[rule_format]: "  S -[f, v, N]\<rightarrow>\<^sub>F S2  ,RL\<Longrightarrow>
((\<forall> C'\<in>cpSet S . (\<forall> f\<in> LocalReferencedRqs C'.
  (RegisteredFuture f (getName C') S0 \<or>(f,(getName C')) \<in>(set RL0)))) \<longrightarrow>
  (\<forall> C'\<in>cpSet S2 . (\<forall> f\<in> LocalReferencedRqs C'.
  (RegisteredFuture f (getName C') S0 \<or>(f,(getName C')) \<in>(set (RL0@RL))))))"
apply (erule Fut.induct);
apply (unfold LocalRegisteredFuturesComp_def  RegisteredFuture_def);
apply (intro ballI allI impI conjI)+;
apply simp+;
apply (case_tac "N=N'");
apply simp+;
(*3*)
apply (erule conjE)+;
apply (erule disjE);
apply (simp add: foldUn)+;
apply (insert UpdFut_futset);
apply (drule_tac x=f in meta_spec);
apply (drule_tac x=v in meta_spec);
apply (erule disjE);
(*5*)
apply (elim bexE conjE);
apply (drule_tac x="parameter b" in meta_spec);
apply (drule subsetD);
apply simp+;
apply (erule disjE);
apply force;
apply (drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply force;
apply blast;
(*4*)
apply (elim bexE conjE);
apply (drule_tac x="fValue b" in meta_spec);
apply (drule subsetD);
apply simp+;
apply (erule disjE);
apply force;
apply (drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply force;
apply blast;
apply blast;
(*2*)
apply simp+;
apply (erule disjE);
apply force;
apply (simp add: cpListset_Un)
apply (elim exE bexE conjE);
apply (subgoal_tac " \<exists> i < length subCp'. C'a=subCp'!i ");
apply (elim exE bexE conjE);
apply (drule_tac x=i in spec)
apply (elim impE)
apply simp+;
(*3*)
apply (elim conjE);
apply (elim impE);
apply (intro ballI allI impI conjI)+;
(*4*)
apply (drule_tac x=C'b in spec);
apply force;
apply force;
apply (force simp: set_conv_nth);
(*1*)
apply (intro ballI allI impI conjI)+;
apply simp+;
apply (case_tac "N=N'");
apply simp+;
apply (simp add: foldUn)+;
apply (drule_tac x=f in meta_spec);
apply (drule_tac x=v in meta_spec);
apply (erule disjE);
apply(subgoal_tac "set (PRqRefs s2) \<subseteq> (set (PRqRefs (PintState s)))-{f}\<union>set (snd v)")
apply (drule subsetD);
apply simp+;
(*4*)
apply (elim conjE disjE);
apply (drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply blast;
apply blast;
defer;

apply (elim bexE disjE);
apply (drule_tac x="parameter b" in meta_spec);
apply (drule subsetD);
apply simp+;
apply (erule disjE);
(*5*)
apply force;
apply (drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply force;
apply blast;
(*3*)
apply (drule_tac x="fValue b" in meta_spec);
apply (drule subsetD);
apply simp+;
apply (erule disjE);
apply force;
apply (drule_tac x=fa in bspec);
apply (simp add: foldUn)+;
apply force;
apply force;
apply simp+;

(*1 behaviour stuff *)
apply (subgoal_tac "  Rep_Behaviours (behaviour s)\<in> Behaviours");
apply (simp add: Behaviours_def);
apply (elim conjE);
 apply (rotate_tac 10);
 apply (drule_tac x="PintState s" in spec);
 apply (rotate_tac -1, drule_tac x=s2 in spec);
apply clarify;
apply (simp add: Behaviour_def);
 apply (drule_tac x=aa in spec);
 apply (drule_tac x=ba in spec);
 apply (drule_tac x=a in spec);
 apply (drule_tac x=b in spec);
apply blast
apply (simp add: Rep_Behaviours);
done;

lemma  registeredFutures_F[rule_format]: " \<lbrakk> S -[f, v, N]\<rightarrow>\<^sub>F S2  ,RL; \<forall> C\<in>cpSet S. LocalRegisteredFuturesComp C S0;
 C'\<in>cpSet S2; f\<in> LocalReferencedRqs C'\<rbrakk>
  \<Longrightarrow>  (RegisteredFuture f (getName C') S0 \<or>(f,(getName C')) \<in>(set RL))"
(* lemma registeredFutures_F:
        \<lbrakk> ?S -[?f,?v,?N]\<rightarrow>\<^sub>F?S2.0,?RL ; ?S \<in> cpSet ?S0.0; \<And>C. C \<in> cpSet ?S \<Longrightarrow> LocalRegisteredFuturesComp C ?S0.0; ?C' \<in> cpSet ?S2.0;
         ?fa \<in> LocalReferencedRqs ?C'\<rbrakk>
        \<Longrightarrow> RegisteredFuture ?fa (getName ?C') ?S0.0 \<or> (?fa, getName ?C') \<in> set ?RL
*)
apply (insert registeredFutures_F_general);
apply (drule_tac x= "S" in meta_spec)
apply (drule_tac x= f in meta_spec)
apply (drule_tac x= v in meta_spec)
apply (drule_tac x= N in meta_spec)
apply (drule_tac x= S2 in meta_spec)
apply (drule_tac x= RL in meta_spec)
apply (drule_tac x= S0 in meta_spec);
apply (drule_tac x= "[]" in meta_spec);
apply (drule_tac x= C' in meta_spec)
apply (drule_tac x= f in meta_spec)
apply (unfold LocalRegisteredFuturesComp_def  RegisteredFuture_def);
apply auto;
done;


lemma registeredFutures_FList_induct[rule_format]:
        " 
  (\<forall> i<len. (SL!i -[f, v,Rcl!i ]\<rightarrow>\<^sub>F (SL!(i+1)), (RLL!i))) \<longrightarrow> (\<forall>C \<in> cpSet (SL!0). LocalRegisteredFuturesComp C (SL!0)) \<longrightarrow>len \<le>length RLL \<longrightarrow>
          (\<forall>C' \<in> cpSet (SL!len). ( \<forall> fa .  fa \<in> LocalReferencedRqs C' \<longrightarrow>
         (RegisteredFuture fa (getName C') (SL!0) \<or> (fa, getName C') \<in> set (concat (take len RLL)))))"
apply (induct_tac len)
apply (unfold LocalRegisteredFuturesComp_def  RegisteredFuture_def);
apply (insert registeredFutures_F_general);
apply force;
apply simp+;
apply (intro impI allI ballI);
apply (drule_tac x= "SL!n" in meta_spec)
apply (drule_tac x= f in meta_spec)
apply (drule_tac x= v in meta_spec)
apply (drule_tac x= "Rcl!n" in meta_spec)
apply (drule_tac x= "SL!(Suc n)" in meta_spec)
apply (drule_tac x= "RLL! n" in meta_spec)
apply (drule_tac x= "SL!0" in meta_spec);
apply (drule_tac x= "concat (take n RLL)" in meta_spec);
apply (drule_tac x= C' in meta_spec)
apply (drule_tac x= "(a,b)" in meta_spec)
apply simp+;
apply (erule meta_impE);
apply (unfold LocalRegisteredFuturesComp_def  RegisteredFuture_def);
apply simp+;
apply (drule_tac x=C'a in bspec);
apply simp+;
apply (rotate_tac 5);
apply (drule_tac x= "fst fa" in spec)
apply (drule_tac x= "snd fa" in spec)
apply simp+;
apply (elim bexE disjE);
apply blast;
apply (rule disjI2);
apply (rule_tac x=x in bexI)
apply blast;
apply (simp add: set_conv_nth)
apply (elim exE);
apply (rule_tac x=i in exI)
apply force;
apply (rule disjI2);
apply (rule_tac x="RLL!n" in bexI)
apply blast;
apply (simp add: set_conv_nth)
apply (rule_tac x=n in exI)
apply force;
done;


lemma F_maintainsRegFutures_pre[rule_format]:
" C -[f,v,Rcl ! i]\<rightarrow>\<^sub>F C',RL \<Longrightarrow> CorrectComponentWeak C' \<longrightarrow>
 RegisteredFuture fa N C \<longrightarrow> RegisteredFuture fa N C'"
apply (erule Fut.induct);
apply (unfold RegisteredFuture_def)
apply auto;
apply (drule_tac CpList.getRecSubCpList_conv_nth);
apply clarify;
apply (drule_tac x= i in spec);
apply auto;
apply (drule CorrectComponentListComp);
apply (drule_tac x="subCp'!i" in CorrectComponentListComp_rev);
apply force;
apply force;
apply (subgoal_tac "i< length subCp'");
apply (rotate_tac -1);
apply (drule CorrectComponentListComp);
apply (drule CorrectComp_getRecSubCpList_conv_nth_rev);
apply simp+;
apply (drule_tac CpList.getRecSubCpList_conv_nth);
apply clarify;
apply (drule_tac x= i in spec);
apply auto;
apply (drule CorrectComponentListComp);
apply (drule_tac x="subCp'!i" in CorrectComponentListComp_rev);
apply force;
apply force;
apply (subgoal_tac "i< length subCp'");
apply (rotate_tac -1);
apply (drule CorrectComponentListComp);
apply (drule CorrectComp_getRecSubCpList_conv_nth_rev);
apply simp+;
done;

lemma F_maintainsRegFutures[rule_format]:
" \<lbrakk>C -[f,v,Rcl ! i]\<rightarrow>\<^sub>F C',RL ; CorrectComponentWeak C ; RegisteredFuture fa N C \<rbrakk> \<Longrightarrow>RegisteredFuture fa N C'"
by (auto simp: F_maintainsRegFutures_pre Fut_maintains_WF);

lemma F_maintainsRegFutures_induct[rule_format]:
        "  (\<forall> i<len. (SL!i -[f, v,Rcl!i ]\<rightarrow>\<^sub>F (SL!(i+1)), (RLL!i))) \<longrightarrow> len \<le>length RLL \<longrightarrow> 
CorrectComponentWeak (SL!0) \<longrightarrow> RegisteredFuture fa N (SL!0) \<longrightarrow> (CorrectComponentWeak (SL!len) \<and>RegisteredFuture fa N (SL!len))"
apply (induct_tac len);
apply auto;
apply (drule_tac x=n in spec);
apply (force simp: Fut_maintains_WF);
apply (drule_tac x=n in spec);
apply (force simp: F_maintainsRegFutures);
done;

lemma registeredFutures_FList:
   " \<lbrakk>\<forall>i < length Rcl .  (SL ! i) -[f,v,Rcl ! i]\<rightarrow>\<^sub>F(SL ! (i + 1)),(RLL ! i) ;length RLL = length Rcl;length SL=(length Rcl)+1; 
         \<forall> C \<in> cpSet (SL ! 0). LocalRegisteredFuturesComp C (SL ! 0); C' \<in> cpSet (SL ! (length Rcl)); fa \<in> LocalReferencedRqs C'; CorrectComponentWeak (SL!0)\<rbrakk>
        \<Longrightarrow> RegisteredFuture fa (getName C') (SL ! length Rcl) \<or> (fa, getName C') \<in> set (concat RLL)"
apply (subgoal_tac  "\<And> i. i < length Rcl \<Longrightarrow>  (SL ! i) -[f,v,Rcl ! i]\<rightarrow>\<^sub>F(SL ! (i + 1)),(RLL ! i) ")
apply (drule registeredFutures_FList_induct)
apply auto;
apply (subgoal_tac  "\<And> i. i < length Rcl \<Longrightarrow>  (SL ! i) -[f,v,Rcl ! i]\<rightarrow>\<^sub>F(SL ! (i + 1)),(RLL ! i) ")
apply (drule F_maintainsRegFutures_induct)
apply auto;
done;

lemma RegisteredFuture_Enqueue[rule_format]:
"CorrectComponentWeak C\<longrightarrow> id R \<notin> set (RqIdList C) \<longrightarrow>(invokedItf R)\<in> dom (getItfs C) \<longrightarrow> kind (the ((getItfs C) (invokedItf R))) = Server \<longrightarrow>RegisteredFuture fa N C \<longrightarrow>         (RegisteredFuture fa N (C \<leftarrow> R))"
apply (unfold RegisteredFuture_def);
apply clarify;
apply simp+;
apply (frule_tac N = "snd fa" and C' = Ca  in getRecSubCp_Some);
apply (case_tac "Ca=C");
apply simp+;
apply (rule_tac x="C\<leftarrow>R" in exI);
apply (rotate_tac -3, frule_tac N = "snd fa" and C' = "C\<leftarrow>R"  in getRecSubCp_Some);
apply (case_tac C);
apply auto;
apply (rule_tac x=Ca in exI);
apply (rotate_tac -3, frule_tac N = "snd fa" and C' = Ca  in getRecSubCp_Some);
apply (case_tac C);
apply auto;
done;

lemma O_maintainsRegFutures[rule_format]:
"S \<turnstile> C -[i1,f,v]\<rightarrow>\<^sub>O C' \<Longrightarrow>
 RegisteredFuture fa N C \<longrightarrow> RegisteredFuture fa N C'"
apply (erule comm.induct);
apply (unfold RegisteredFuture_def);
apply auto;
done;

lemma RegFuture_upd:
"\<lbrakk>CorrectComponentWeak (Composite N itfs sub bind s);
 CorrectComponentWeakList (sub <-C');
RegisteredFuture f N' (Composite N itfs sub bind s);
sub ^ (getName C') = Some C;
\<forall>f N. (RegisteredFuture f N C\<longrightarrow>RegisteredFuture f N C')\<rbrakk>
\<Longrightarrow> RegisteredFuture f N' (Composite N itfs (sub<-C') bind s)"
apply (unfold RegisteredFuture_def);
apply simp+;
apply auto;
apply (drule_tac x="fst f" in spec);
apply (drule_tac x="snd f" in spec);
apply (drule_tac x=N' in spec);
apply simp+;
(*1*)
apply (drule CorrectComponentListComp);
apply (drule_tac C'=Ca and N="snd f" in getRecSubCpList_conv_nth);
apply (frule_tac getCp_conv_nth);
apply clarify;
apply (case_tac "i=ia");
apply simp+;
apply (erule exE);
apply (rule_tac x=Ca in exI);
apply simp+;
(*2*)
apply (rule getRecSubCpList_upd);
apply simp+;
apply (rule_tac x=C in exI);
apply simp+;
(*1*)
apply (subgoal_tac "sub!i=(sub<-C')!i");
apply (rule_tac i=i in CorrectComp_getRecSubCpList_conv_nth_rev);
apply force;
apply force;
apply simp+;
apply (rule sym,rule upd_diff_nth);
apply simp;
(*1*)
apply (drule correctCompWeakList_distinct_localNames)+;
apply (drule getCp_inlist)+;
apply (simp add: distinct_conv_nth);
apply force;
done;

lemma O_newFuture:
"S \<turnstile> C -[i1,f,v]\<rightarrow>\<^sub>O C' \<Longrightarrow>f\<notin>set (RqIdList S)"
apply (erule_tac comm.cases);
apply simp+;
done;

lemma R_maintainsRegFutures_pre[rule_format]:
"S\<turnstile> C \<rightarrow>\<^sub>R C',RL \<Longrightarrow> C\<in>cpSet S \<longrightarrow> CorrectComponentWeak C \<longrightarrow> CorrectComponentWeak C' \<longrightarrow>
 RegisteredFuture fa N C \<longrightarrow> RegisteredFuture fa N C'"
apply (erule red.induct);
apply (unfold RegisteredFuture_def);
apply clarify;
apply (frule getRecSubCp_Some_pre1_Cp);
apply simp+;
apply (simp split: split_if_asm);
apply simp+;
apply clarify;
apply (drule CorrectComponentListComp);

apply (subgoal_tac "CorrectComponentWeakList (subCp <-c2)");
apply (frule_tac N = "snd fa" and C' = Ca  in getRecSubCpList_Some);
apply simp+;
apply (elim conjE)
apply (drule CpList.cpListset_cpSet);
apply (elim bexE);
apply (case_tac "C'=C")
apply clarify;
apply simp+;
apply (drule_tac x=C in CorrectComponentListComp_rev);
apply force;
apply simp+;
apply (erule impE);
apply (rule_tac C'="Composite N0 itf subCp bindings s" in cpSetcpSet);
apply force;
apply simp;
apply (erule impE);
apply (rule red_maintains_WF);
apply simp+;
apply (rotate_tac -2,frule_tac N = "snd fa" and C' = Ca  in getRecSubCp_Some);
apply simp+;
apply clarify;
apply (rule_tac x=Cb in exI);
apply simp+;
apply (frule_tac N = "snd fa" and C' = Cb in getRecSubCpList_Some);
apply simp+;
apply (subgoal_tac "CorrectComponentWeak c2");
apply (drule_tac N = "snd fa" and C=c2 and C'=Cb  in getRecSubCp_Some);
apply simp+;
apply (rule_tac C'=c2 in setcpSet_cpListset);
apply (subgoal_tac "subCp^Na\<noteq> None");
apply (drule_tac C=c2 in upd_SameName);
apply (drule red_maintains_name);
apply simp+;
apply (drule getCp_name);
apply force;
apply (simp add: getCp_inlist);
apply simp+;
apply (rule red_maintains_WF);
apply simp+;
apply (rule_tac x=Ca in exI);
apply (rotate_tac -6,frule_tac N = "snd fa" and C' = Ca in getRecSubCpList_Some);
apply simp+;
apply (rule_tac C'=C' in setcpSet_cpListset);
apply (rule upd_diff);
apply simp+;
apply (drule red_maintains_name);
apply simp+;
apply (drule correctCompWeakList_distinct_localNames)+;
apply (simp add: distinct_conv_nth set_conv_nth)+;
apply (drule  getCp_conv_nth);
apply clarify;
apply (drule_tac x=ib in spec)+;
apply simp+;
apply (drule_tac x=i in spec)+;
apply simp+;
apply force;
(*3*)

apply (fold RegisteredFuture_def);
apply clarify;
apply (frule_tac C'=c2 in RegFuture_upd);
apply simp+;
apply (rule_tac C=C in CorrectComponentWeakList_update);
apply (drule comm_maintains_name);
apply (frule getCp_name, simp);
apply force;
apply (erule comm_maintains_WF);
apply (rule_tac subCp=subCp in CorrectComponentListComp_rev);
apply force;
apply force;
apply (drule comm_names_eq);
apply (rule_tac subCp=subCp in CorrectComponentListComp_rev);
apply force;
apply force;
apply force;

apply (drule O_sameRqIdList)
apply force;

apply (subgoal_tac "CorrectComponentWeak c2");
apply (force simp: CorrectComponentWeak_def)
apply (erule comm_maintains_WF);
apply (rule_tac subCp=subCp in CorrectComponentListComp_rev);
apply force;
apply force;
apply simp+;
apply (drule comm_maintains_name);
apply (frule getCp_name, simp);
apply clarify;
apply (drule O_maintainsRegFutures);
apply simp+;

(*4*)
apply (subgoal_tac "CorrectComponentWeak  (Composite N0 itf (subCp <- c2) bindings s)")
apply (rotate_tac -2,frule_tac C=C' and C'="C' \<leftarrow> \<lparr>Request.id = (a, b), parameter = (aa, ba), invokedItf = i2\<rparr>" in RegFuture_upd);
apply auto;
apply (rotate_tac 4, frule getCp_name)
apply (case_tac C');apply force;apply force;
apply (subgoal_tac "CorrectComponentWeak C'", rotate_tac -1);

apply (rotate_tac -1,drule_tac  R="\<lparr>Request.id = (a, b), parameter = (aa, ba), invokedItf = i2\<rparr>" in RegisteredFuture_Enqueue);
apply simp+;

apply (frule O_newFuture);
apply (subgoal_tac "set (RqIdList C') \<subseteq> set (RqIdList S)");
apply force;
apply (subgoal_tac "\<exists>C. subCp^(getName C') = Some C" );
apply (elim exE);
apply (subgoal_tac " set (RqIdList C') =  set (RqIdList Ca)");
apply simp;
apply (rule cpSetRqId)
apply (subgoal_tac "Ca\<in>cpSet (Composite N0 itf subCp bindings s)");
apply (erule  cpSetcpSet);
apply force;
apply force;
apply (case_tac "Ca=C");
apply (frule O_sameRqIdList);
apply (subgoal_tac "C'=c2");
apply force;
apply (frule comm_maintains_name);
apply simp+;
apply (subgoal_tac "Na=b");
apply (subgoal_tac "(subCp<-c2)^b=Some c2");
apply force;
apply (rule getCpI);
apply force;
apply (drule getCp_name)+;
apply force;
apply (rotate_tac -5, frule getCp_name);
apply force;
apply (drule getCp_name)+;
apply force;
apply (subgoal_tac "Ca=C'");
apply force;
apply (subgoal_tac "b\<noteq>getName c2")
apply (drule_tac CL=subCp in upd_getCpunchanged);
apply simp+;
apply (rotate_tac 5, frule getCp_name);
apply force;
apply (subgoal_tac "getName Ca\<noteq>getName C")
apply (drule getCp_name)+;
apply (drule comm_maintains_name);
apply (subgoal_tac "getName Ca\<noteq>getName C")
apply simp;
apply force;
apply (drule CorrectComponentListComp)+;
apply (rotate_tac -2);
apply (drule correctCompWeakList_distinct_localNames);
apply (drule getCp_inlist)+;
apply clarify;
apply (simp add: distinct_conv_nth set_conv_nth);
apply clarify;
apply (rotate_tac 8);
apply (drule_tac x=i in spec);
apply force;
apply force;
apply (simp+);
(*9*)
(*once to prove its an interface *)
apply (thin_tac "CorrectComponentWeak
         (Composite N0 itf ((subCp <- c2) <- C' \<leftarrow> \<lparr>Request.id = (a, b), parameter = (aa, ba), invokedItf = i2\<rparr>)
           bindings s)");
apply (thin_tac "CorrectComponentWeak (Composite N0 itf subCp bindings s)");
apply (subgoal_tac "CorrectComponentWeakList (subCp<-c2)")
apply (simp add: CorrectComponentWeak_def);
apply clarify;
apply (drule_tac x="\<lparr>src = Na .\<^sub>i i1, dest = b .\<^sub>i i2\<rparr>" in bspec);
apply force;
apply ((elim conjE)+,simp+);
apply force;

(*7 and again to prove itis of the good kind *)
apply (thin_tac "CorrectComponentWeak
         (Composite N0 itf ((subCp <- c2) <- C' \<leftarrow> \<lparr>Request.id = (a, b), parameter = (aa, ba), invokedItf = i2\<rparr>)
           bindings s)");
apply (thin_tac "CorrectComponentWeak (Composite N0 itf subCp bindings s)");
apply (subgoal_tac "CorrectComponentWeakList (subCp<-c2)")
apply (simp add: CorrectComponentWeak_def);
apply clarify;
apply (drule_tac x="\<lparr>src = Na .\<^sub>i i1, dest = b .\<^sub>i i2\<rparr>" in bspec);
apply force;
apply ((elim conjE)+,simp+);
apply force;
apply simp+;
(*4*)
apply (drule getCp_inlist)+;
apply (rule_tac subCp="subCp<-c2" in CorrectComponentListComp_rev);
apply force;
apply force;
(*3*)
apply (thin_tac "CorrectComponentWeak
         (Composite N0 itf ((subCp <- c2) <- C' \<leftarrow> \<lparr>Request.id = (a, b), parameter = (aa, ba), invokedItf = i2\<rparr>)
           bindings s)");
apply (unfold CorrectComponentWeak_def);
apply simp+;
apply (subgoal_tac "CorrectComponentWeakList (subCp<-c2)")
apply (rule);
apply clarify;
apply rule;
apply (drule_tac x=bb in bspec)
apply simp+;
apply (case_tac "src bb");
apply force;
apply simp+;
apply (frule_tac comm_maintains_name);
apply (case_tac "getName c2 = Name1");
apply (subgoal_tac "subCp ^ Name1\<noteq>None");
apply (drule_tac C=c2 in upd_SameName);
apply simp+;
apply (drule_tac N=Name2 in comm_maintains_itfs); 
apply (force simp: getCp_inlist);
apply force;
(*6*)
apply (drule not_sym,drule_tac CL=subCp in upd_getCpunchanged);
apply simp+;
apply (case_tac "dest bb");
apply force;
apply simp+;
apply (case_tac "getName c2 = Name1");
apply (frule_tac comm_maintains_name);
apply (subgoal_tac "(subCp <- c2) ^ Name1 = Some c2" );
apply simp+;
apply (drule_tac N=Name2 in comm_maintains_itfs); 
apply simp+;
apply (drule_tac x=bb in bspec)
apply force;
apply clarify;apply simp+;
apply (force simp: getCp_inlist);
apply (subgoal_tac "subCp ^ Name1\<noteq>None");
apply (drule_tac C=c2 in upd_SameName);
apply simp+;
apply (force simp: getCp_inlist);
(*5*)
apply (drule not_sym,drule_tac CL=subCp in upd_getCpunchanged);
apply simp+;
apply (drule_tac x=bb in bspec)
apply assumption;
apply force;
(*4*)
apply rule;
apply (unfold CorrectComponentWeakList_def);
apply force;
apply simp+;
apply (subgoal_tac "subCp^(getName c2) = Some C")
apply rule;
apply (drule upd_RqIdList);
apply (unfold CorrectComponentWeakList_def);
apply force;
apply simp+;
apply (drule O_sameRqIdList);
apply simp+;
apply (subgoal_tac "set(RqIdList C)\<subseteq>set(RqIdListList subCp)")
apply (rotate_tac -1);
apply blast;
apply (rule cpSetRqIdList);
apply force;
apply force;
apply rule;
(*6 again*)
apply (drule upd_RqIdList);
apply (unfold CorrectComponentWeakList_def);
apply force;
apply simp+;
apply (drule O_sameRqIdList);
apply simp+;
apply (subgoal_tac "set(RqIdList C)\<subseteq>set(RqIdListList subCp)")
apply (subgoal_tac "set(RqIdListList subCp)\<union>set(RqIdList C) = set(RqIdListList subCp)")
apply simp;
apply force;
apply (rule cpSetRqIdList);
apply force;
apply force;
apply (fold cpListset_def)
apply (drule upd_cpset);
apply (unfold CorrectComponentWeakList_def);
apply force;
apply simp+;
apply (drule comm_names_eq);
apply (subgoal_tac "CorrectComponentWeakList subCp")
apply (rule CorrectComponentListComp_rev);
apply (simp add: getCp_inlist)+;
apply (unfold CorrectComponentWeakList_def);
apply force;
apply simp+;
apply (simp add: image_Un)+;
apply (subgoal_tac "cpSet  C\<subseteq>cpListset  subCp")
apply force;
apply (rule getCp_cpSet);
apply force;
apply (frule_tac comm_maintains_name);
apply (force simp: getCp_inlist);
apply (fold CorrectComponentWeakList_def CorrectComponentWeak_def)
(*3*)
apply (subgoal_tac "CorrectComponentWeakList subCp")
apply (rule_tac C=C in CorrectComponentWeakList_update);
apply (frule_tac comm_maintains_name);
apply (force simp: getCp_inlist);
apply simp;
apply (rule comm_maintains_WF);
apply force;
apply (drule getCp_inlist);
apply (rule CorrectComponentListComp_rev);
apply simp+;
(*6*)
apply (drule comm_names_eq);
apply (rule CorrectComponentListComp_rev);
apply simp+;
apply force;
apply force;
(*5*)
apply (drule O_sameRqIdList);
apply force;
apply (drule O_sameRqIdList);
apply (subgoal_tac "CorrectComponentWeak C");
apply (force simp: CorrectComponentWeak_def);
apply (rule CorrectComponentListComp_rev);
apply force;apply (force simp: getCp_inlist);
(*3*)
apply (force simp: CorrectComponentWeakList_def);
apply (unfold RegisteredFuture_def);
apply simp+;
apply rule;
apply force;
apply clarify;
apply simp+;
apply (drule getRecSubCpList_existssubCp);
apply (clarify);
apply (case_tac "Cb=C");
apply (frule_tac fa=fa and N=N in O_maintainsRegFutures);
apply (unfold RegisteredFuture_def);
apply force;
apply clarify;
apply (rule_tac x=Cc in exI);
apply simp+;
apply (subgoal_tac "c2\<in> set (subCp <- c2)",rotate_tac -1);
apply (simp add: set_conv_nth);apply clarify;
apply (rule CorrectComp_getRecSubCpList_conv_nth_rev);
apply force;
apply force;
apply force;
(*3*)
apply rule;
apply (drule comm_maintains_name);
apply (simp add: getCp_inlist);
apply (rule_tac x=Ca in exI);
apply simp+;
apply (subgoal_tac "Cb\<in> set (subCp <- c2)",rotate_tac -1);
apply (simp add: set_conv_nth);apply clarify;
apply (rule CorrectComp_getRecSubCpList_conv_nth_rev);
apply force;
apply force;
apply force;
apply (rule upd_diff);
apply (drule getCp_inlist)+;
apply force;
apply (drule getCp_inlist)+;
apply (drule comm_maintains_name);
apply (subgoal_tac "CorrectComponentWeakList subCp");
apply (drule correctCompWeakList_distinct_localNames);
apply simp+;
apply (simp add: distinct_conv_nth set_conv_nth);
apply force;
apply force;
(*1*)
apply (fold RegisteredFuture_def);
apply (rule_tac C=C' in RegFuture_upd);
apply (force simp: CorrectComponentWeak_def);
apply force;
apply (force simp: RegisteredFuture_def);
apply (simp add: getCp_inlist);
apply clarify;
apply (rule RegisteredFuture_Enqueue);
apply (rule CorrectComponentListComp_rev);
apply force;
apply force;
apply simp+;
(*4*)
apply (subgoal_tac "C'\<in>cpSet S");
apply (drule cpSetRqId)+;
apply force;
apply (rule_tac C'="Composite Na itf subCp bindings s " in cpSetcpSet)
apply force;
apply force;
apply (force simp: CorrectComponentWeak_def);
apply (force simp: CorrectComponentWeak_def);
apply simp+;
done;

lemma R_maintainsRegFutures[rule_format]:
" \<lbrakk>S\<turnstile> C \<rightarrow>\<^sub>R C',RL ; CorrectComponentWeak C ; RegisteredFuture fa N C; C\<in>cpSet S\<rbrakk> 
\<Longrightarrow>RegisteredFuture fa N C'"
by (auto simp: R_maintainsRegFutures_pre red_maintains_WF);


theorem registeredFutures: "\<turnstile> C1 \<leadsto> C2 \<Longrightarrow> 
((CorrectComponent C1 \<and>GlobalRegisteredFuturesComp C1) \<longrightarrow>GlobalRegisteredFuturesComp C2)"
apply (erule red2.induct);
apply (intro impI, elim conjE)
apply (rule futures_are_registered_in_S2)
apply (unfold GlobalRegisteredFuturesComp_def)
defer;
apply (drule registeredFutures_FList)
apply simp+;
apply (rule CorrectCompWeak);
apply simp+;
apply (fold GlobalRegisteredFuturesComp_def);
apply (intro impI, elim conjE)
apply (rule futures_are_registered_in_S2)
defer;
apply (frule_tac C'=C' in registeredFutures_R)
apply (fold cpSet_def); apply force;
apply (unfold GlobalRegisteredFuturesComp_def)
apply simp+;
apply (elim disjE);
apply (drule R_maintainsRegFutures);
apply (erule CorrectCompWeak);
apply simp+;
apply force;
apply simp+;
apply (subgoal_tac "CorrectComponent(SL ! length Rcl)");
apply (intro conjI);
apply (rule CorrectCompWeak);
apply simp;
defer;
apply (subgoal_tac "\<forall>i<length Rcl. CorrectComponent (SL ! (Suc i))" )
apply (drule_tac x= "(length Rcl) - 1" in spec)+;
apply simp+;
apply (subgoal_tac " \<forall>N.  ((\<forall>i<N.  SL ! i -[f,v,Rcl ! i]\<rightarrow>\<^sub>FSL ! Suc i,RLL ! i)\<longrightarrow>(\<forall>i<N. CorrectComponent (SL ! Suc i)))")
apply force;
apply (intro allI);
apply (induct_tac N);
apply simp+;
apply (intro allI impI);
apply (case_tac "i<n");
apply simp+;
apply (subgoal_tac "i=n");
apply (rotate_tac -4,drule_tac x= "n" in spec);
apply (drule_tac x= "n -  1" in spec)+;
apply simp+;
apply (case_tac n);
apply simp+;
apply (drule fut_maintains_WF_Strong);
apply simp+;
apply (drule fut_maintains_WF_Strong);
apply force;
apply force;
apply force;

apply (subgoal_tac "CorrectComponent S1");
apply (intro conjI);
apply (erule CorrectCompWeak);
defer;
apply (rule red_maintains_WF_Strong);
apply simp+;
defer;
apply (unfold CorrectComponent_def);
apply simp+;
apply clarify;
apply (rotate_tac -2, drule_tac x="(a,b)" in bspec);
apply force;
apply (simp add: cpSet_def)+;

apply clarify;
apply (rotate_tac -2, drule_tac x="(ab,bb)" in bspec);
apply force;
apply (simp add: cpSet_def)+;
done;



(*primrec RegisteredFuturesComp::"Component \<Rightarrow> Component\<Rightarrow> bool" and 
   RegisteredFuturesCompList::"Component List\<Rightarrow> Component\<Rightarrow> bool"
"RegisteredFuturesComp (Primitive N itfs bindings s) S \<equiv>
  ( \<forall>f\<in> LocalReferencedRqs (Primitive N itfs bindings s). RegisteredFuture f N S)" |
"RegisteredFuturesComp (Composite N itfs subCp bindings s) S \<equiv>
  ( \<forall>f\<in> LocalReferencedRqs (Composite N itfs subCp bindings s). RegisteredFuture f N S)\<and>
  RegisteredFuturesCompList subCp S" |
*)
end;


